//-----------------------------------------------------------------------------
//  Copyright (C) 2008 Thomas S. Ullrich 
//
//  This file is part of "eDISK".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//  
//  Author: Thomas S. Ullrich
//  Last update: Feb 10, 2008
//-----------------------------------------------------------------------------
#ifndef MAINWINDOW_H
#define MAINWINDOW_H
#include <QMainWindow>
#include <QtWidgets>
#include <vector>

#define PR(x) cout << #x << " = " << (x) << endl;
#define VERSION 1.04
#define VERSIONSTRING "1.04"
 
using namespace std;

class QAction;
class QGraphicsView;
class QGraphicsScene;
class QLineEdit;
class QComboBox;
class QTableWidget;
class QGraphicsPixmapItem;
class QGraphicsLineItem;
class QDoubleSpinBox;
class QComboBox;
class QGraphicsPolygonItem;
class QGraphicsTextItem;
class QGraphicsEllipseItem;
class QLabel;
class eDISKHelpBrowser;
class eDISKinematics;
class eDISKHelpBrowser;

class eDISKWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    eDISKWindow();
    
protected:    
    void closeEvent(QCloseEvent*);
    
private slots:
        
    // File menu
    void print();
    void finish();
    
    // View
    void setAngularUnits();
        
    // Help menu
    void about();
    void help();
    
    void setMode(int);
    void recalculateAndUpdate();

    
private:
    void createGraphicsScene();
    void createMenuActions();
    void createMenus();
    void createStatusBar();
    void createDockingWindow();
    void createDockingWindowElements(QWidget*);
    void createHelpBrowser();

    void drawScene();

    void connectAll();
    void disconnectAll();
        
protected:
    void fitInView();
    bool eventFilter(QObject*, QEvent*);
    bool event(QEvent*); 
    
private:    
    enum {mXY, mXQ2, mYQ2, mElectron, mQuark};
    
    QGraphicsView   *mImageView;
    QGraphicsScene  *mImageScene;
    
    QMenu *mFileMenu;
    QMenu *mViewMenu;
    QMenu *mHelpMenu;
    
    QDockWidget *mKinematicsDock;
    
    QAction *mPrintAction;
    QAction *mFinishAction;
    QAction *mAboutAction;
    QAction *mHelpAction;
    QAction *mViewRadiansAction;
    
    QComboBox *mModeSelectionComboBox;

    QDoubleSpinBox *mField_Ebeam_e;
    QDoubleSpinBox *mField_Ebeam_p;  
    QLineEdit      *mField_sqrtS;
    QDoubleSpinBox *mField_y;  
    QLineEdit      *mField_x;
    QDoubleSpinBox *mField_Q2;
    QLineEdit      *mField_Ee;
    QDoubleSpinBox *mField_ae;
    QLineEdit      *mField_Eq;
    QDoubleSpinBox *mField_aq;
    QLineEdit      *mField_nu;
    QLineEdit      *mField_W;
    
    QRectF            mViewport;
    
    QGraphicsLineItem *mBeamline;
    QGraphicsLineItem *mVerticalAxis;
    QGraphicsLineItem *mElectronDirectionLine;    
    QGraphicsLineItem *mProtonDirectionLine;    
    QGraphicsLineItem *mScatteredElectronLine;
    QGraphicsLineItem *mScatteredQuarkLine;
    
    QGraphicsPolygonItem *mElectronDirectionArrow;    
    QGraphicsPolygonItem *mProtonDirectionArrow;    
    
    QGraphicsTextItem *mElectronDirectionText;    
    QGraphicsTextItem *mProtonDirectionText;    
    
    QGraphicsEllipseItem *mAngleIndicator;
    vector<QGraphicsLineItem*> mAngleIndicatorTics;
    
    QPointF mScatteredElectronTip;
    QPointF mScatteredQuarkTip;
    
    bool            mRadianUnits;
    
    QLabel *mLabelElectronAngle;
    QLabel *mLabelQuarkAngle;
    
    eDISKHelpBrowser *mHelpBrowser;
    eDISKinematics   *mEngine;

};

#endif
